"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var bn_js_1 = tslib_1.__importDefault(require("bn.js"));
var contract_1 = require("../contract");
var address_1 = require("../address");
var dposv2_pb_1 = require("../proto/dposv2_pb");
var big_uint_1 = require("../big-uint");
var DPOS2 = /** @class */ (function (_super) {
    tslib_1.__extends(DPOS2, _super);
    function DPOS2(params) {
        return _super.call(this, params) || this;
    }
    DPOS2.createAsync = function (client, callerAddr) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var contractAddr;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, client.getContractAddressAsync('dposV2')];
                    case 1:
                        contractAddr = _a.sent();
                        if (!contractAddr) {
                            throw Error('Failed to resolve contract address');
                        }
                        return [2 /*return*/, new DPOS2({ contractAddr: contractAddr, callerAddr: callerAddr, client: client })];
                }
            });
        });
    };
    DPOS2.prototype.getCandidatesAsync = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var listCandidatesReq, result;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        listCandidatesReq = new dposv2_pb_1.ListCandidateRequestV2();
                        return [4 /*yield*/, this.staticCallAsync('ListCandidates', listCandidatesReq, new dposv2_pb_1.ListCandidateResponseV2())];
                    case 1:
                        result = _a.sent();
                        return [2 /*return*/, result.getCandidatesList().map(function (candidate) { return ({
                                address: address_1.Address.UmarshalPB(candidate.getAddress()),
                                pubKey: candidate.getPubKey_asU8()
                            }); })];
                }
            });
        });
    };
    DPOS2.prototype.getValidatorsAsync = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var listValidatorReq, result;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        listValidatorReq = new dposv2_pb_1.ListValidatorsRequestV2();
                        return [4 /*yield*/, this.staticCallAsync('ListValidators', listValidatorReq, new dposv2_pb_1.ListValidatorsResponseV2())];
                    case 1:
                        result = _a.sent();
                        return [2 /*return*/, result.getValidatorsList().map(function (validator) { return ({
                                pubKey: validator.getPubKey_asU8(),
                                power: validator.getPower()
                            }); })];
                }
            });
        });
    };
    DPOS2.prototype.checkDelegationAsync = function (validator, delegator) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var checkDelegationReq, result, delegation;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        checkDelegationReq = new dposv2_pb_1.CheckDelegationRequestV2();
                        checkDelegationReq.setValidatorAddress(validator.MarshalPB());
                        checkDelegationReq.setDelegatorAddress(delegator.MarshalPB());
                        return [4 /*yield*/, this.staticCallAsync('CheckDelegation', checkDelegationReq, new dposv2_pb_1.CheckDelegationResponseV2())];
                    case 1:
                        result = _a.sent();
                        delegation = result.getDelegation();
                        return [2 /*return*/, delegation
                                ? {
                                    validator: address_1.Address.UmarshalPB(delegation.getValidator()),
                                    delegator: address_1.Address.UmarshalPB(delegation.getDelegator()),
                                    height: new bn_js_1.default(delegation.getHeight()),
                                    amount: delegation.getAmount() ? big_uint_1.unmarshalBigUIntPB(delegation.getAmount()) : new bn_js_1.default(0)
                                }
                                : null];
                }
            });
        });
    };
    DPOS2.prototype.delegateAsync = function (validator, amount) {
        var delegateRequest = new dposv2_pb_1.DelegateRequestV2();
        delegateRequest.setValidatorAddress(validator.MarshalPB());
        delegateRequest.setAmount(big_uint_1.marshalBigUIntPB(new bn_js_1.default(amount)));
        return this.callAsync('Delegate', delegateRequest);
    };
    DPOS2.prototype.unbondAsync = function (validator, amount) {
        var unbondRequest = new dposv2_pb_1.UnbondRequestV2();
        unbondRequest.setValidatorAddress(validator.MarshalPB());
        unbondRequest.setAmount(big_uint_1.marshalBigUIntPB(new bn_js_1.default(amount)));
        return this.callAsync('Unbond', unbondRequest);
    };
    return DPOS2;
}(contract_1.Contract));
exports.DPOS2 = DPOS2;
//# sourceMappingURL=dpos2.js.map